/*
  META & LINKS REPORT
  Test meta tags in your pages
  Works on a local image of the site.
  A free open-source Scriptol program by D.G. Sureau
  http://www.scriptol.net
  Licence: absolutely free to use, just keep the above references
*/

include "path.sol"
include "libphp.sol"

void usage()
  print "Meta & Links Report 1.6 - www.scriptol.net"
  print "Display metas inside head section of html files and links"
  print "usage:  metarep [option] filename"
  print "options:"
  print "   -m   displays meta tags only"
  print "   -l   displays links only"
  print "   -g   output a log file rather than screen"
  exit(1)
return

array pageList = ()
text pageText
text basePath
text baseHost = ""
boolean METAFLAG
boolean LINKFLAG
boolean LOGFLAG
array parsedList = ()
array logfile = ()
text logname          ` path and name of the log file

void display(text t)
   if LOGFLAG
      logfile.push(t + "\n")
   else
      print t
   /if
return

` test if the address is included into

boolean hasHost(text theurl)
   if "http://" in theurl ? return true
   if "ftp://"  in theurl ? return true
return false


` get the internet address

text getHost(text theurl)
   boolean f1 = false
   int offset = 0
   offset = theurl.find("http://")
   if offset = nil
      offset = theurl.find("ftp://")
      f1 = true
   /if
   if offset = nil ? return ""
   offset + 6
   if not f1 ? offset + 1

   text sub = theurl[offset..]
   offset = sub.find("/", offset)
   if offset = nil ? return theurl       ` no file or subdir
return theurl[--offset]


` get the part of tag inside quotes or double-quotes, thus the url

text extractLink(int off)
   text c
   text link = ""
   while off < pageText.length()
     c = pageText[off]
     if c = '='  ? continue
     if c = ' '  ? continue
     if c = '"'  : off + 1; break; /if
     if c = '\'' : off + 1; break; /if
     break
   let off + 1
   while off < pageText.length()
     c = pageText[off]
     if c = '\'' ? break
     if c = '"'  ? break
     if c = '>'  ? break
     link + c             ` anything else is part of the url
   let off + 1
return link.lower().trim()


` Get the URL from the text, skipping other chars

array getLinks(constant alias text fname)
  array x = ()
  x.load(basePath + fname)
  //print "getlink in " , basePath + fname
  array links = ()
  array pageLinks = ()
  ` removing ending special codes
  scan x let x[] = x[].toText().rtrim().lower()
  pageText = x.join("")            ` merging lines into a text
  int offset = 0
  int srcoff = 0
  while forever
    srcoff = offset
    offset = pageText.find("href=", offset)
    if offset = nil ? offset = pageText.find("src=", srcoff)
    if offset = nil ? break
    offset + 4
    text link = extractLink(offset)
    if link = "" ? continue
    if link[0] = '#' ? continue
    if link in links ? continue
    if link in pageLinks ? continue
    pageLinks.push(link)
    if hasHost(link)
       text host = getHost(link)
       if host != baseHost
          if LINKFLAG ? display("External link:" + link)
          continue
       /if
    /if
    if LINKFLAG ? display( "Link:" + link)
    if link in pageList ? continue
    links.push(link)
    pageList.push(link)
  /while
return links


` parse a web page

int metaPage(text fname)
  int counter = 0
  text fullpath

  text node, ext
  node, ext = Path.splitExt(fname)
  ` only html pages are parsed for meta tags
  if ext.lower() not in ("htm", "html", "php", "php3", "php4", "php5", "asp",
    "shtml", "dhtml", "jsp") ? return 0

  fullpath = basePath + fname
  if file_exists(fullpath)
    display("")
    display(fullpath)
    display("-".dup(fullpath.length()))
    dict x = get_meta_tags(fullpath)
    text k, v
    if METAFLAG
       if x.size() > 0
         for k, v in x
           display(k.upper() + " : " + v)
         /for
       else display("*** No any meta tag ***")
       /if
    /if
    parsedList.push(fname.lower().trim())
    counter + 1
    array dep = getLinks(fname)
    if dep = nil ? return counter
    for text page in dep
      counter + metaPage(page)
    /for
  else
    display(fname + " no found")
  /if

return counter


` read an ".ini" file, parse lines and return a dict of options

void readIni(text inipath)

  array ini
  inipath = Path.changeExt(inipath, "ini")
  if not file_exists(inipath) return
  ini.load(inipath)
  if(ini.size() = 0)
    display(inipath + " file not loaded")
    return
  /if

  for text option in ini
    option = option.trim()        ` remove trailing spaces and eol
    int i = option.find("=")      ` parse a line
    if i = nil  continue          ` no valid couple key=value
    text k = option[--i].trim()   ` key
    text v = option[i+1..].trim() ` value
    if k
    = "logfile": logname = v
    /if
  /for

return


int main(int num, array args)
  text fname
  int fileindex

  logname = Path.changeExt(args[0], "log")

  if num not in (2,3) let usage()
  METAFLAG = true
  LINKFLAG = true
  LOGFLAG  = false
  fileindex = 1


  if num = 3
    text option = args[1]
    fileindex = 2
    if (option[0] <> '-') and (option[0] <> '/') let usage()
    if option[1].lower()
    = "l" : METAFLAG = false
    = "m" : LINKFLAG = false
    = "g" : LOGFLAG  = true
    else usage()
    /if
  /if

  display("Meta & Links Report. A Scriptol program.")

  readIni(args[0])

  int counter = 0
  text fpath = args[fileindex].toText()
  if not Path.exists(fpath)
    display(fpath + " not found")
    if LOGFLAG let logfile.store(logname)
    exit()
  /if


  basePath, fname = Path.splitFile(fpath)
  if hasHost(basePath)
     baseHost = getHost(basePath)
     if baseHost.length() = basePath.length() ? fname = fpath
  /if
  pageList.push(fname.lower().trim())
  counter  + metaPage(fname)
  display("")
  parsedList.sort()
  display("List of parsed files:")
  scan parsedList let display(parsedList[])
  display(text(counter) + " pages")

  if LOGFLAG let logfile.store(logname)

return 0

main($argc, $argv)
